/*
 * Copyright (c) 2011-2012 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE.txt file for licensing information.
 */
package eu.unicore.ucc.ccm;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import junit.framework.Assert;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import de.fzj.unicore.ucc.UCC;

import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;
import xmlbeans.eu.unicore.ccm.internal.UCCExtensionType;

public class PluginsUpdaterTest
{
	@Test
	public void testInstall() throws FileNotFoundException, IOException
	{
		ClientProfile profile = ClientProfile.Factory.newInstance();
		UCCExtensionType ext1 = profile.addNewUccExtension();
		ext1.addCommand("cmd1=ue.unicore.Test1");
		ext1.addCommand("cmd2=ue.unicore.Test2");
		ext1.setPluginName("TestPlugin");
		ext1.setDescription("TestPlugin description");
		ext1.setMaxClientVersion("1.10.0");
		ext1.setMinClientVersion("1.4.2");
		ext1.setPluginVersion("1.0.3");
		ext1.setPluginURL("file:src/test/resources/plugin1-fake.jar");
		
		install(profile, "src/test/resources/extensions1", false);
	}

	private void install(ClientProfile profile, String extensions, boolean update) throws FileNotFoundException, IOException
	{
		File workdir = new File("target/test-workdir");
		if (workdir.exists())
			FileUtils.deleteQuietly(workdir);
		workdir.mkdir();
		File libdir = new File(workdir, "lib");
		
		PluginsUpdater updater = new PluginsUpdater(new MockMessageWriter(), 
			workdir.toString());
		UCC.unitTesting = true;
		PluginsUpdater.unitTestVersion = "1.4.3-SNAPSHOT"; 
		
		try
		{
			updater.processPlugins(profile, extensions, update);
		} catch (CCMException e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		
		Properties p = new Properties();
		if (update)
		{
			p.load(new FileInputStream(new File(extensions)));
		} else
			p.load(new FileInputStream(
				new File(workdir, new File(extensions).getName())));
		for (UCCExtensionType ext: profile.getUccExtensionArray())
		{
			Assert.assertTrue(new File(libdir, ext.getPluginName()+"-"+
					ext.getPluginVersion()+".jar").exists());
			for (String cmd: ext.getCommandArray())
			{
				String aa[] = cmd.split("=");
				Assert.assertTrue(p.containsKey(aa[0]));
				Assert.assertTrue(p.getProperty(aa[0]).equals(aa[1]));
			}
		}
	}
}
