package de.fzj.unicore.ucc.actions;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.TransferControllerClient;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.UnitParser;

/**
 * check the status of a asynchronous copy-file operation (@see CopyFile})
 */
public class CopyFileStatus extends ActionBase {

	protected Location sourceDesc;
	protected Location targetDesc;
	protected TransferControllerClient tcc;
	
	protected StorageClient sms;
	
	protected boolean synchronous;
	protected UnitParser up;
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
	}

	@Override
	public void process() {
		super.process();
		
		try{
			up=UnitParser.getCapacitiesParser(3);
			initTCC();
			verbose("Transfer Resource "+tcc.getEPR().getAddress().getStringValue()
					+" contacted, reply: servertime="+tcc.getCurrentTime());
			doCheck();
		}catch(Exception e){
			error("Can't copy file.",e);
			endProcessing(ERROR);
		}
	}
	
	protected void initTCC()throws Exception{
		String arg=null;
		if(getCommandLine().getArgs().length>0)arg=getCommandLine().getArgs()[1];
		EndpointReferenceType epr=null;
		if(arg==null){
			//read from stdin
			try{
				String url=new BufferedReader(new InputStreamReader(System.in)).readLine();
				epr=EndpointReferenceType.Factory.newInstance();
				epr.addNewAddress().setStringValue(url);
			}catch(Exception e){
				System.err.println("Can't read file transfer descriptor from stdin.");
				endProcessing();
			}
		}
		else{
			if(new File(arg).exists()){
				epr=EndpointReferenceType.Factory.parse(new File(arg));
			}
			else{
				epr=EndpointReferenceType.Factory.newInstance();
				epr.addNewAddress().setStringValue(arg);
			}
		}
		tcc=new TransferControllerClient(epr.getAddress().getStringValue(),epr,securityProperties);
	}
	
	protected void doCheck()throws Exception{
		long size=0;
		long transferred=-1;
		size=tcc.getSize();
		transferred=tcc.getTransferredBytes();
		String status=tcc.getStatus();
		message(status+", ["+up.getHumanReadable(transferred)
				+" / "+up.getHumanReadable(size)+"] bytes transferred");
	}
	
	@Override
	public String getName() {
		return "copy-file-status";
	}

	@Override
	public String getSynopsis() {
		return "Checks the status of a copy-file operation.";
	}
	
	@Override
	public String getDescription(){
		return "check status of a copy-file";
	}
	
	@Override
	public String getArgumentList(){
		return "[IdFile|Address]";
	}
	@Override
	public String getCommandGroup(){
		return "Data management";
	}
}
