package de.fzj.unicore.ucc.actions;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.unigrids.services.atomic.types.StatusType;
import org.unigrids.services.atomic.types.StatusType.Enum;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.ucc.util.Builder;

public class GetStatus extends ActionBase {

	static String lastStatus;
	static boolean allSuccessful;
	
	protected JobClient createJobClient(String arg){
		JobClient job=null;
		Builder builder=createBuilder(arg);
		String url=builder.getProperty("epr");
		if(url==null){
			error("Job EPR not found! Maybe <"+arg+"> has not been produced by ucc.",null);
			endProcessing(ERROR_CLIENT);
		}
		try{
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(url);
			job = new JobClient(url,epr,securityProperties);
			job.getCurrentTime();
			verbose("Job ID OK.");
		}
		catch(Exception e){
			error("Can't contact job",e);
			endProcessing(ERROR);
		}
		return job;
	}

	/**
	 * create a {@link Builder} instance
	 * @param arg - denoting either a file, or a URL
	 */
	protected Builder createBuilder(String arg){
		Builder builder=null;
		try{
			File job=new File(arg);
			if(job.exists())
			{
				builder=new Builder(job);
				builder.setCheckLocalFiles(false);
				verbose("Read job from <"+arg+">");
			}
			else{
				verbose("Accessing job at EPR = <"+arg+">");
				builder=new Builder();
				builder.setCheckLocalFiles(false);
				builder.setProperty("epr", arg);
			}
		}catch(Exception e){
			error("Can't use <"+arg+">.",e);
			endProcessing(ERROR_CLIENT);
		}
		return builder;
	}

	@Override
	public void process(){
		super.process();
		allSuccessful=true;
		
		List<String> args=new ArrayList<String>();
		
		if(getCommandLine().getArgs().length==1){
			try{
				String arg=new BufferedReader(new InputStreamReader(System.in)).readLine();
				args.add(arg);
			}catch(Exception e){
				error("Can't read job descriptor from stdin.",e);
				endProcessing(ERROR_CLIENT);
			}	
		}
		else{
			for(int i=1; i<getCommandLine().getArgs().length;i++){
				args.add(getCommandLine().getArgs()[i]);
			}
		}
		
		for(String arg: args){
			JobClient job=createJobClient(arg);
			getStatus(job);	
		}
		
	}
	
	@Override
	public String getName(){
		return "get-status";
	}
	
	@Override
	public String getArgumentList(){
		return "[<jobfile> | <jobepr>]";
	}
	@Override
	public String getSynopsis(){
		return "Gets the status of a UNICORE 6 job. " +
				"The job is referenced either by a job file as written " +
				"by the 'run' command or as an EPR.";
	}
	@Override
	public String getDescription(){
		return "get job status";
	}
	@Override
	public String getCommandGroup(){
		return "Job execution";
	}

	protected void getStatus(JobClient job){
		try{
			Enum status=job.getStatus();
			String url=job.getEPR().getAddress().getStringValue();
			StringBuilder sb=new StringBuilder();
			sb.append(url).append(" ");
			sb.append(status);
			if(!status.equals(StatusType.SUCCESSFUL)){
				allSuccessful=false;
			}
			if(status.equals(StatusType.SUCCESSFUL)){
				Integer exit=job.getExitCode();
				if(exit!=null)sb.append(" exit code: "+exit);
			}
			if(status.equals(StatusType.RUNNING)){
				Float progress=job.getProgress();
				if(progress!=null)sb.append(" progress: "+(int)(100*progress)+"%");
			}
			lastStatus=sb.toString();
			
			message(sb.toString());
		}catch(Exception e){
			error("Can't get job status.",e);
			endProcessing(ERROR);
		}
	}
	
}
