package de.fzj.unicore.ucc.actions;

import org.apache.commons.cli.OptionBuilder;
import org.unigrids.x2006.x04.services.tss.ApplicationResourceType;

import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.helpers.TargetSystemLister;

public class ListApplications extends ListActionBase<TSSClient> {
	
	protected String siteName;

	public String getName(){
		return "list-applications";
	}
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Site Name")
				.withArgName("Vsite")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);
	}

	@Override
	public void process() {
		super.process();
		
		siteName=getCommandLine().getOptionValue(OPT_SITENAME);
		
		for(TSSClient c: new TargetSystemLister(registry,securityProperties,this)){
			try{
				String tName=c.getTargetSystemName();
				if(siteName!=null){
					if(!siteName.equals(tName))continue;
				}
				if(filterMatch(c)){
					verbose("Listing TargetSystem <"+tName+"> at "+c.getEPR().getAddress().getStringValue());
					message("Applications on target system <"+tName+">");
					listApps(c);
				}
			}catch(Exception ex){
				String msg="Error accessing TSS at "+c.getUrl();
				logger.error(msg,ex);
				verbose(msg);
			}
		}
	}

	protected void listApps(TSSClient tss){
		for(ApplicationResourceType app: tss.getApplications()){
			String out=app.getApplicationName()+" "+app.getApplicationVersion();
			if(app.getDescription()!=null)out+=" ("+app.getDescription()+")";
			message(out);
			lastNumberOfResults++;
		}		
	}
	
	@Override
	public String getDescription(){
		return "lists applications on target systems";
	}
	
	
	@Override
	public String getSynopsis(){
		return "Lists the available applications per target system. " +
				"The list can be limited to a single target system specified " +
				"using the '-s' option.";
	}
	@Override
	public String getArgumentList(){
		return "";
	}
	@Override
	public String getCommandGroup(){
		return "General";
	}
}
