package de.fzj.unicore.ucc.actions;

import org.apache.commons.cli.OptionBuilder;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.helpers.JobLister;
import de.fzj.unicore.ucc.helpers.TargetSystemLister;

/**
 * creates a job listing
 * 
 * @author schuller
 */
public class ListJobs extends ListActionBase<JobClient> {
	
	protected String siteName;

	public String getName(){
		return "list-jobs";
	}
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Site Name")
				.withArgName("Vsite")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);
	}

	@Override
	public void process() {
		super.process();
		siteName=getCommandLine().getOptionValue(OPT_SITENAME);
		
		for(TSSClient c: new TargetSystemLister(registry,securityProperties,this)){
			String tName=c.getTargetSystemName();
			if(siteName!=null){
				if(!siteName.equals(tName))continue;
			}
			verbose("Listing TargetSystem <"+tName+"> at "+c.getEPR().getAddress().getStringValue());
			for(JobClient job: new JobLister(c)){
				if(filterMatch(job)){
					listJob(job);
					lastNumberOfResults++;
				}
			}
		}
	}

	protected void listJob(JobClient job){
		message(job.getEPR().getAddress().getStringValue()+getDetails(job));
		printWSRFProperties(job);
	}
	

	@Override
	protected String getDetails(JobClient job){
		if(!detailed)return "";
		
		try	{
			return " "+job.getStatus();
		}catch(Exception ex){
			return " ERROR getting status";
		}
	}
	
	@Override
	public String getDescription(){
		return "list your jobs";
	}
	
	@Override
	public String getSynopsis(){
		return "Lists your jobs per target system. " +
				"The list can be limited to a single target system specified " +
				"using the '-s' option.";
	}
	
	@Override
	public String getArgumentList(){
		return "";
	}
	@Override
	public String getCommandGroup(){
		return "General";
	}
}
