package de.fzj.unicore.ucc.actions;

import java.util.Arrays;

import org.apache.commons.cli.OptionBuilder;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.RangeValueType;
import org.unigrids.services.atomic.types.AvailableResourceType;
import org.unigrids.services.atomic.types.AvailableResourceTypeType;
import org.unigrids.x2006.x04.services.tss.ApplicationResourceType;
import org.unigrids.x2006.x04.services.tss.TargetSystemPropertiesDocument.TargetSystemProperties;

import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.helpers.TargetSystemLister;
import eu.unicore.jsdl.extensions.ExecutionEnvironmentDescriptionDocument.ExecutionEnvironmentDescription;

public class ListSites extends ListActionBase<TSSClient> {

	protected String siteName;

	public String getName(){
		return "list-sites";
	}
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Site Name")
				.withArgName("Vsite")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);
	}
	
	@Override
	public void process() {
		super.process();
		siteName=getCommandLine().getOptionValue(OPT_SITENAME);
		for(TSSClient c: new TargetSystemLister(registry,securityProperties,this)){
			if(filterMatch(c)){
				 try{
					 if(siteName!=null){
							if(!siteName.equals(c.getTargetSystemName()))continue;
						}
					 listTSS(c);
					 lastNumberOfResults++;
				 }catch(Exception ex){
					 error("Error listing site at <"+c.getEPR().getAddress().getStringValue()+">",ex);
				 }
			}
		}
	}

	protected void listTSS(TSSClient tss)throws Exception{
		message(tss.getTargetSystemName()+" "+tss.getEPR().getAddress().getStringValue()+" "+getDetails(tss));
		printWSRFProperties(tss);
	}
	
	@Override
	protected String getDetails(TSSClient tss)throws Exception{
		if(!detailed)return "";
		StringBuilder details=new StringBuilder();
		String sep=System.getProperty("line.separator");
		details.append(sep).append("  Operating system: ").append(tss.getOperatingSystemInfo());
		details.append(sep).append("  Number of jobs: ").append(tss.getNumberOfJobs());
		
		boolean first=true;
		for(ApplicationResourceType a: tss.getApplications()){
			if(!first){
				details.append(", ");
			}
			else {
				details.append(sep).append("  Applications: ");
				first=false;
			}
			details.append(a.getApplicationName()).append(" ").append(a.getApplicationVersion());
		}
		TargetSystemProperties tssProps=tss.getResourcePropertiesDocument().getTargetSystemProperties();
		first=true;
		for(ExecutionEnvironmentDescription ee:tssProps.getExecutionEnvironmentDescriptionArray()){
			if(!first){
				details.append(", ");
			}
			else{
				details.append(sep).append("  Execution environments: ");
				first=false;
			}
			details.append(ee.getName());
			if(ee.getVersion()!=null)details.append(" ").append(ee.getVersion());
		}
		
		String[] xlogins=tss.getXlogins();
		if(xlogins!=null && xlogins.length>0){
			details.append(sep).append("  Xlogins: "+Arrays.asList(xlogins));
		}
		
		String[] xgroups=tss.getXgroups();
		if(xgroups!=null && xgroups.length>0){
			details.append(sep).append("  Xgroups: "+Arrays.asList(xgroups));
		}
		
		listResources(tssProps, details);
		
		return details.toString();
	}

	protected void listResources(TargetSystemProperties tssProps, StringBuilder details){
		String sep=System.getProperty("line.separator");
		try{
			String nodes=getRange(tssProps.getTotalResourceCount());
			if(nodes!=null){
				details.append(sep).append("  Nodes: ").append(nodes);
			}
			String cpusPerNode=getRange(tssProps.getIndividualCPUCount());
			if(cpusPerNode!=null){
				details.append(sep).append("  CPUsPerNode: ").append(cpusPerNode);
			}
			String cpus=getRange(tssProps.getTotalResourceCount());
			if(cpus!=null){
				details.append(sep).append("  CPUs: ").append(cpus);
			}
			//non-jsdl
			AvailableResourceType[] aR=tssProps.getAvailableResourceArray();
			for(AvailableResourceType a: aR){
				details.append(sep).append("  ").append(a.getName()).append(": ").append(getRange(a));
			}
		}
		catch(Exception ex){
			
		}
	}
	
	private String getRange(RangeValueType rv){
		if(rv==null)return null;
		String lower=rv.getRangeArray()[0].getLowerBound().getStringValue();
		String upper=rv.getRangeArray()[0].getUpperBound().getStringValue();
		return lower+" - "+upper;
	}
	
	private String getRange(AvailableResourceType res){
		if(res==null)return null;
		if(AvailableResourceTypeType.INT == res.getType() || AvailableResourceTypeType.DOUBLE == res.getType()){
			String lower=res.getMin();
			String upper=res.getMax();
			return "["+lower+" - "+upper+"]";
		}
		else if (AvailableResourceTypeType.CHOICE == res.getType()){
			return String.valueOf(Arrays.asList(res.getAllowedValueArray()));
		}
		else if (AvailableResourceTypeType.STRING== res.getType()){
			return "string-valued";
		}
		else if (AvailableResourceTypeType.BOOLEAN== res.getType()){
			return "true|false";
		}
		else{
			return "";
		}
	}
	
	@Override
	public String getDescription(){
		return "list remote sites";
	}

	@Override
	public String getArgumentList(){
		return "";
	}
	@Override
	public String getCommandGroup(){
		return "General";
	}
	
	@Override
	public String getSynopsis() {
		return "List the Grid sites available to you, together " +
				"with some information about the site's capabilities. "
				+"Use the -l or -a options to control the amount of information. "
				+"Use the -s option to limit the list to a single site.";
	}

	private static String lastSiteAddress;

	public static String getLastSiteAddress() {
		return lastSiteAddress;
	}

	protected static void setLastSiteAddress(String lastAddress) {
		ListSites.lastSiteAddress = lastAddress;
	}
}
