package de.fzj.unicore.ucc.actions;

import java.util.Arrays;

import javax.xml.namespace.QName;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.RangeValueType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.StorageManagement;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.helpers.StorageLister;
import de.fzj.unicore.ucc.helpers.TargetSystemLister;
import de.fzj.unicore.ucc.util.UnitParser;
import eu.unicore.util.httpclient.IClientConfiguration;

public class ListStorages extends ListActionBase<StorageClient> implements IServiceInfoProvider{

	public String getName(){
		return "list-storages";
	}
	
	protected UnitParser unitParser=UnitParser.getCapacitiesParser(1);
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
	}

	@Override
	public void process() {
		super.process();
		for(TSSClient c: new TargetSystemLister(registry,securityProperties,this)){
			for(StorageClient sms: new StorageLister(c,securityProperties,this)){
				if(filterMatch(sms)){
					 listSMS(sms);
					 lastNumberOfResults++;
				}
			};
		}
		for(StorageClient sms: new StorageLister(registry,securityProperties,this)){
			if(filterMatch(sms)){
				 listSMS(sms);
				 lastNumberOfResults++;
			}
		};
	}

	protected void listSMS(StorageClient sms){
		try{
			message(sms.getStorageName()+" "+sms.getEPR().getAddress().getStringValue()+getDetails(sms));
		}catch(Exception ex){
			error("Error listing storage at "+sms.getEPR().getAddress().getStringValue(), ex);
		}
		printWSRFProperties(sms);
	}
	
	@Override
	protected String getDetails(StorageClient sms) {
		if(!detailed)return "";
		String sep=System.getProperty("line.separator");
		StringBuilder sb=new StringBuilder();
		try{
			RangeValueType diskSpace=sms.getFileSystem().getDiskSpace();
			if(diskSpace!=null && diskSpace.getExactArray()!=null && diskSpace.getExactArray().length>0){
				double space=diskSpace.getExactArray()[0].getDoubleValue();
				sb.append(sep).append("  Available: ").append(unitParser.getHumanReadable(space));
			}
		}catch(Exception ex){}
		try{
			sb.append(sep).append("  Protocols: ").append(Arrays.asList(sms.getSupportedProtocols()));
			sb.append(sep).append("  Metadata: ").append(sms.supportsMetadata()?"yes":"no");
			sb.append(sep).append("  ACLs: ").append(sms.supportsACL()?"yes":"no");
		}catch(Exception ex){}
		
		return sb.toString();
	}

	@Override
	public String getDescription(){
		return "list the available remote storages";
	}
	@Override
	public String getArgumentList(){
		return "";
	}
	@Override
	public String getCommandGroup(){
		return "General";
	}
	
	public QName getQName(){
		return StorageManagement.SMS_PORT;
	}
	
	public String getServiceName(){
		return "Storage";
	}
	
	public String getServiceDetails(EndpointReferenceType epr, IClientConfiguration securityProperties){
		//TODO
		return null;
	}
}
