package de.fzj.unicore.ucc.actions;

import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.UCC;

/**
 * print info about services available in the registry
 */
public class SystemInfo extends ActionBase {


	private boolean details=false;
	
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt("long")
				.withDescription("Detailed output.")
				.isRequired(false)
				.create("l")
				);
	}
	
	@Override
	public String getName() {
		return "system-info";
	}
	
	
	@Override
	public String getDescription() {
		return "check the availability of services";
	}

	@Override
	public String getCommandGroup(){
		return "General";
	}
	
	@Override
	public String getSynopsis() {
		return "Checks the registry for the availability of UNICORE services. " +
				"If the '-l' option is used, the address of the found service is printed";
	}


	@Override
	public void process() {
		super.process();
		details=getBooleanOption("long", "l");
		getInfo();
	}
	
	protected void getInfo(){
		//ucc commands may optionally get info about services they access
		for(Command c: UCC.getAllCommands()){
			if(! (c instanceof IServiceInfoProvider) ){
				continue;
			}
			IServiceInfoProvider info=(IServiceInfoProvider)c;
			QName port=info.getQName();
			String name=info.getServiceName();
			message("");
			message("Checking for "+name+" service ...");
			try{
				List<EndpointReferenceType>list=registry.listServices(port);
				int n=list.size();
				if(n==0){
					message("... no services available");
				}
				else{
					message("... OK, found "+n+" service(s)");
					if(details){
						for(EndpointReferenceType e: list){
							message(" * "+e.getAddress().getStringValue());
							String infoS=info.getServiceDetails(e,securityProperties);
							if(infoS!=null)message("   "+infoS);
						}
					}
				}
			}catch(Exception e){
				error("... FAILED.",e);
			}
		}
	}	
}
