package de.fzj.unicore.ucc.actions;

import java.util.Calendar;
import java.util.Date;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.BaseUASClient;
import de.fzj.unicore.ucc.util.UnitParser;

/**
 * low-level wsrf operations
 *
 * @author schuller
 */
public class WSRF extends ActionBase {
	protected BaseUASClient wsrf;
	
	private static String lastProperties;
	
	@Override
	public void process(){
		super.process();
		try{
			int length=getCommandLine().getArgs().length;
			if(length<2){
				throw new IllegalArgumentException("You must provide at least a WSRF command as argument.");
			}
			String cmd=getCommandLine().getArgs()[1];
			if(length<3){
				throw new IllegalArgumentException("You must provide a URL as argument to this command.");
			}
			//URL is last argument
			String url=getCommandLine().getArgs()[length-1];
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(url);
			wsrf=new BaseUASClient(url,epr,securityProperties);
			verbose("Connecting to <"+url+">");
			if("getproperties".startsWith(cmd))doGetProperties();
			else if("destroy".startsWith(cmd))doDestroy();
			else if("extend".startsWith(cmd)){
				Integer days=Integer.parseInt(getCommandLine().getArgs()[2]);
				doExtendLifetime(days);
			}
			else if("setterminationtime".startsWith(cmd)){
				doSetTT();
			}
			
			else {
				throw new IllegalArgumentException("Command <"+cmd+"> not understood!");
			}

		}catch(Exception e){
			error("Can't perform wsrf operation", e);
			endProcessing(ERROR);
		}
	}

	@Override
	protected boolean requireRegistry() {
		return false;
	}

	protected void doGetProperties() throws Exception{
		String props=wsrf.getResourcePropertyDocument();
		message(props);
		lastProperties=props;
	}
	
	protected void doDestroy()throws Exception{
		wsrf.destroy();
		verbose("Destroyed resource.");
	}
	
	protected void doSetTT()throws Exception{
		String dString=getCommandLine().getArgs()[2];
		Date d=UnitParser.extractDateTime(dString);
		Calendar newTT=Calendar.getInstance();
		newTT.setTime(d);
		wsrf.setTerminationTime(newTT);
		verbose("Set termination time to '"+dString+"'");
	}
	
	/**
	 * extend the lifetime
	 * @param days - the number of days to add to the termination time
	 * @throws Exception
	 */
	protected void doExtendLifetime(int days)throws Exception{
		Calendar tt=wsrf.getTerminationTime();
		tt.add(Calendar.DATE, days);
		wsrf.setTerminationTime(tt);
		verbose("Extended termination time by <"+days+"> days.");
	}
	
	public String getName(){
		return "wsrf";
	}
	
	public String getArgumentList(){
		return "[getproperties | destroy | extend <days> | setterminationtime <'yyyy-MM-dd HH:mm'|'HH:mm'> ] URL";
	}
	
	public String getSynopsis(){
		return "Perform a lowlevel WSRF operation such as getProperties() " +
				"or destroy() on the resource at <URL>";
	}
	
	public String getDescription(){
		return "perform a WSRF operation";
	}
	
	static String getLastProperties(){
		return lastProperties;
	}
	
}
