package de.fzj.unicore.ucc.authn;

import java.util.Properties;
import java.util.ServiceLoader;

import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * This interface encapsulates the various ways that a user can "log in" to UCC<br/>
 * 
 * Implementations are discovered at runtime using the {@link ServiceLoader} mechanism.
 * The user can choose the mechanism using a commandline option
 * 
 * @author schuller
 * @since 1.6.0
 */
public interface Authentication {

	/**
	 * returns the name of the AuthN mechanism. Users can select the AuthN mechanism
	 * easily by simply specifying the name.
	 */
	public String getName();
	
	/**
	 * Returns a description of the AuthN mechanism
	 */
	public String getDescription();
	
	/**
	 * the main method of this interface is used to retrieve a working 
	 * {@link IClientConfiguration}
	 */
	public IClientConfiguration initSecurityProperties(Properties uccPreferences)throws Exception;

	/**
	 * print human-readable usage info (e.g. config properties and their description)
	 * @param out
	 */
	public void printUsage();
}
