package de.fzj.unicore.ucc.helpers;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.TSSClient;

/**
 * Convenient access to the jobs on a given target system
 *  
 * @author schuller
 */
public class JobLister implements Iterable<JobClient>{

	private static final Logger logger=Logger.getLogger(JobLister.class.getName());
	
	private TSSClient tss;
	
	public JobLister(TSSClient tss){
		this.tss=tss;
	}
	
	/**
	 * returns an iterator over the available jobs, providing a
	 * pre-initialised JobClient per job
	 * 
	 * @see de.fzj.unicore.uas.client.JobClient
	 */
	public Iterator<JobClient> iterator(){
		try{
			final List<EndpointReferenceType> eprs=tss.getJobs();

			return new Iterator<JobClient>(){
				
				Iterator<EndpointReferenceType> iter=eprs.iterator();
				public boolean hasNext() {
					return iter.hasNext();
				}
				public JobClient next() {
					EndpointReferenceType epr=iter.next();
					try{
						return new JobClient(epr.getAddress().getStringValue(),epr,tss.getSecurityConfiguration());
					}catch(Exception e){
						logger.error("Can't create Job Client.",e);
						return null;
					}
				}

				public void remove() {
					iter.remove();
				}
			};
		}catch(Exception e){
			logger.error("",e);
		}
		return null;
	}

}
