package de.fzj.unicore.ucc.helpers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.TSFClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * provides access to the available target systems
 * 
 * @author schuller
 */
public class TargetSystemLister implements Iterable<TSSClient>{

	private final IRegistryQuery registry;
	private final IClientConfiguration sec;
	private final MessageWriter msg;

	public TargetSystemLister(IRegistryQuery registry, IClientConfiguration sec, MessageWriter msg){
		this.registry=registry;
		this.sec=sec;
		this.msg=msg;
	}


	/**
	 * return a list of all accessible target systems, 
	 * providing a pre-initialised TSSClient per instance
	 * @return a (possibly empty) list of accessible TSSs
	 */
	public List<TSSClient> list(){
		TargetSystemFactoryLister tsfLister=new TargetSystemFactoryLister(registry,sec,msg);
		List<TSSClient> tssList=new ArrayList<TSSClient>();
		for(TSFClient tsf: tsfLister){
			try{
				List<EndpointReferenceType> eprs=tsf.getAccessibleTargetSystems();
				if(eprs!=null){
					for(EndpointReferenceType epr: eprs){
						try{
							TSSClient tss=new TSSClient(epr.getAddress().getStringValue(),epr,sec);
							tssList.add(tss);
						}catch(Exception ex){
							msg.verbose("Can't access target system at "+epr.getAddress().getStringValue());
						}
					}
					if(eprs.size()==0){
						msg.verbose("No accessible target systems at "+tsf.getEPR().getAddress().getStringValue());
					}
				}
				
			}catch(Exception ex){
				String message="Can't access target systems at "+tsf.getEPR().getAddress().getStringValue();
				msg.error(message, ex);
			}
		}
		return tssList;
	}

	/**
	 * return an iterator over the available target systems, providing a
	 * pre-initialised TSSClient per instance
	 * 
	 * @see de.fzj.unicore.uas.client.TSSClient
	 */
	public Iterator<TSSClient> iterator(){
		return list().iterator();
	}

	/**
	 * get a TSS by its site name
	 * 
	 * @param sitename
	 */
	public TSSClient getTSS(String sitename){
		for(TSSClient c: this){
			if(sitename.equalsIgnoreCase(c.getTargetSystemName()))return c;
		}
		return null;
	}



}
