package de.fzj.unicore.ucc.runner;

import org.unigrids.x2006.x04.services.tss.ApplicationResourceType;
import org.unigrids.x2006.x04.services.tss.TargetSystemPropertiesDocument.TargetSystemProperties;

public class ApplicationRequirement implements Requirement {

	private final String appName, appVersion;
	
	public ApplicationRequirement(String appName, String appVersion){
		this.appName=appName;
		this.appVersion=appVersion;
	}
	
	public String getDescription() {
		return "Application: "+appName+(appVersion!=null?" v"+appVersion:"");
	}

	public boolean isFulfilled(TargetSystemProperties tss) {
		ApplicationResourceType[] apps=tss.getApplicationResourceArray();
		for(ApplicationResourceType app: apps){
			if(!appName.equalsIgnoreCase(app.getApplicationName()))continue;
			if(appVersion==null)return true;
			if(appVersion.equalsIgnoreCase(app.getApplicationVersion()))return true;
		}
		return false;
	}

	@Override
	public int hashCode() {
		int result = 1;
		result = 31 * result + ((appName == null) ? 0 : appName.hashCode());
		result = 31 * result
				+ ((appVersion == null) ? 0 : appVersion.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ApplicationRequirement other = (ApplicationRequirement) obj;
		if (appName == null) {
			if (other.appName != null)
				return false;
		} else if (!appName.equals(other.appName))
			return false;
		if (appVersion == null) {
			if (other.appVersion != null)
				return false;
		} else if (!appVersion.equals(other.appVersion))
			return false;
		return true;
	}

}
