package de.fzj.unicore.ucc.runner;

import org.unigrids.x2006.x04.services.tss.TargetSystemPropertiesDocument.TargetSystemProperties;

import eu.unicore.jsdl.extensions.ExecutionEnvironmentDescriptionDocument.ExecutionEnvironmentDescription;

public class ExecutionEnvironmentRequirement implements Requirement {

	private final String name;
	private final String version;
	
	public ExecutionEnvironmentRequirement(String name, String version){
		this.name=name;
		this.version=version;
	}
	
	public String getDescription() {
		return "Execution environment: "+name+(version!=null?" v"+version:"");
	}

	public boolean isFulfilled(TargetSystemProperties tss) {
		ExecutionEnvironmentDescription[] ee=tss.getExecutionEnvironmentDescriptionArray();
		for(ExecutionEnvironmentDescription execEnv: ee){
			if(!name.equalsIgnoreCase(execEnv.getName()))continue;
			if(version==null)return true;
			if(version.equalsIgnoreCase(execEnv.getVersion()))return true;
		}
		return false;
	}

	@Override
	public int hashCode() {
		int result = 1;
		result = 31 * result + ((name== null) ? 0 : name.hashCode());
		result = 31 * result
				+ ((version== null) ? 0 : version.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ExecutionEnvironmentRequirement other = (ExecutionEnvironmentRequirement) obj;
		if (name == null) {
			if (other.name!= null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (version == null) {
			if (other.version != null)
				return false;
		} else if (!version.equals(other.version))
			return false;
		return true;
	}

}
