/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 20-09-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package de.fzj.unicore.ucc.actions;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;

import org.junit.Test;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.impl.tss.TargetSystemFactoryImpl;
import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.util.EmbeddedWithUvos;
import de.fzj.unicore.wsrflite.Resource;
import de.fzj.unicore.wsrflite.Service;

public class TestSAMLPush extends EmbeddedWithUvos
{

	protected void connect(){
		UCC.unitTesting = true;
		UCC.main(new String[]{"connect", "-J", "https://localhost:62774",
				"-G", "/test-vo",
				"-c","src/test/resources/uas-saml/userprefs.embedded"});
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}
	
	protected void disconnect() {
		try
		{
			Service tsfSrv = uas.getKernel().getService("TargetSystemFactoryService");
			TargetSystemFactoryImpl r = (TargetSystemFactoryImpl) tsfSrv.getHome().get("default_target_system_factory");
			EndpointReferenceType eprs[] = r.getTSSReferenceProperty().getProperty();
			for (EndpointReferenceType epr: eprs)
			{
				String addr = epr.getAddress().getStringValue();
				String id = addr.substring(addr.lastIndexOf("?") + 5);
				Resource tss = uas.getKernel().getService("TargetSystemService").getHome().get(id);
				tss.postActivate();
				tss.destroy();
			}
		} catch (Exception e)
		{
			e.printStackTrace();
			fail(e.toString());
		}
	}
	
	
	@Test
	public void testListAttributes() {
		UCC.unitTesting = true;
		String[] args=new String[]{"list-attributes", "-v", 
				"-J", "https://localhost:62774",
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@Test
	public void testDateWithPush() {
		connect();
		try {
			String[] args=new String[]{"run",
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
				"src/test/resources/jobs/date.u",
			};
			UCC.main(args);
		} finally {
			disconnect();
		}
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@Test
	public void testDateWithPush2() {
		connect();
		try {
			String[] args=new String[]{"run",
				"-J", "https://localhost:62774",
				"-G", "/test-vo",
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
				"src/test/resources/jobs/date.u",
			};
			UCC.main(args);
		} finally {
			disconnect();
		}
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@Test
	public void testSavingAssertion() throws IOException {
		UCC.unitTesting = true;
		File tmpAssertion = File.createTempFile("ucc-test", ".xml");
		tmpAssertion.deleteOnExit();
		String[] args=new String[]{"save-attributes",
				"-J", "https://localhost:62774",
				"-G", "/test-vo",
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
				"-O", tmpAssertion.getAbsolutePath(),
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		assertTrue("Fetched assertion has not been saved to file.", 0L!=tmpAssertion.length());
		assertTrue(tmpAssertion.canRead());

		File tmpAssertion2 = File.createTempFile("ucc-test", ".xml");
		args=new String[]{"save-attributes",
				"-A", tmpAssertion.getAbsolutePath(),
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
				"-O", tmpAssertion2.getAbsolutePath(),
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		assertTrue("Assertion changes during saving.", tmpAssertion2.length()==tmpAssertion.length());
	}

	@Test
	public void testSavingAndUsingAssertion() throws IOException {
		UCC.unitTesting = true;
		File tmpAssertion = File.createTempFile("ucc-test", ".xml");
		tmpAssertion.deleteOnExit();
		String[] args=new String[]{"save-attributes",
				"-J", "https://localhost:62774",
				"-G", "/test-vo",
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
				"-O", tmpAssertion.getAbsolutePath(),
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		assertTrue("Fetched assertion has not been saved to file.", 0L!=tmpAssertion.length());
		assertTrue(tmpAssertion.canRead());

		args=new String[]{"connect",
				"-A", tmpAssertion.getAbsolutePath(),
				"-c", "src/test/resources/uas-saml/userprefs.embedded",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		disconnect();
	}
}
