package de.fzj.unicore.ucc.helpers;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;
import static org.junit.Assert.assertTrue;

import java.util.HashMap;
import java.util.Map;

import org.junit.Test;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.util.EmbeddedTestBase;
import de.fzj.unicore.wsrflite.admin.ResourceAvailability;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import de.fzj.unicore.wsrflite.xmlbeans.registry.RegistryCreator;
import de.fzj.unicore.wsrflite.xmlbeans.sg.Registry;

public class TestConnector extends EmbeddedTestBase {

	@Test
	public void testConnector()throws Exception{
		Connector c=makeConnector();
		try{
			c.setInitialLifetime(-1);
			fail("Negative lifetime should be forbidden");
		}catch(IllegalArgumentException e){/*expected*/}
		c.setInitialLifetime(1);
		try{
			c.setTimeout(-1);
			fail("Negative timeout should be forbidden");
		}catch(IllegalArgumentException e){/*expected*/}
		c.setTimeout(10);
		c.run();
		assertEquals(1,c.getAvailableTSS());
		assertEquals(1,c.getAvailableTSF());
	}
	
	@Test
	public void test_SiteUnavailable()throws Exception{
		//switch TSF to unavailable
		toggleAvailable("default_target_system_factory");
		
		long start=System.currentTimeMillis();
		Connector c=makeConnector();
		c.run();
		assertEquals(0,c.getAvailableTSF());
		assertEquals(0,c.getAvailableTSS());
		
		//this should not take much more than the default 10 secs
		assertTrue(start-System.currentTimeMillis()<12*1000);
		
		//re-enable
		toggleAvailable("default_target_system_factory");
	}

	@Test
	public void test_SiteBlacklisted() throws Exception{
		Connector c=makeConnector();
		c.setBlacklist("localhost");
		c.run();
		assertEquals(0,c.getAvailableTSF());
		assertEquals(0,c.getAvailableTSS());
	}

	private Connector makeConnector()throws Exception{
		EndpointReferenceType regEPR=WSServerUtilities.makeEPR(Registry.REGISTRY_SERVICE, 
				RegistryCreator.DEFAULT_REGISTRY_ID, uas.getKernel());
		RegistryClient reg=new RegistryClient(regEPR,uas.getKernel().getClientConfiguration());
		return new Connector(reg,reg.getSecurityConfiguration(),this);
	}
	
	private void toggleAvailable(String id){
		Map<String,String>p=new HashMap<String, String>();
		p.put("resources", id);
		System.out.println(new ResourceAvailability().invoke(p, uas.getKernel()).getMessage());
	}
}
