package de.fzj.unicore.ucc.runner;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.List;
import java.util.Properties;

import org.junit.Test;
import org.unigrids.services.atomic.types.ProtocolType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.TSFClient;
import de.fzj.unicore.ucc.runner.Runner;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.ucc.util.EmbeddedTestBase;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import de.fzj.unicore.wsrflite.xmlbeans.registry.RegistryCreator;
import de.fzj.unicore.wsrflite.xmlbeans.sg.Registry;

public class TestRunner extends EmbeddedTestBase {

	@Test
	public void testAsync(){
		try{
			EndpointReferenceType regEPR=WSServerUtilities.makeEPR(Registry.REGISTRY_SERVICE, 
					RegistryCreator.DEFAULT_REGISTRY_ID, uas.getKernel());
			EndpointReferenceType tsfEPR=WSServerUtilities.makeEPR(UAS.TSF, "default_target_system_factory", uas.getKernel());
			TSFClient tsf=new TSFClient(tsfEPR, uas.getKernel().getClientConfiguration());
			String address=tsf.createTSS().getEPR().getAddress().getStringValue();
			message("Created tss at "+address);
			
			RegistryClient reg=new RegistryClient(regEPR, uas.getKernel().getClientConfiguration());
			Builder b=new Builder().
				setApplication("Date", "1.0").
				writeOutputTo("target/testdata");
			String state=null;
			Properties p=new Properties();
			p.put("protocols","UFTP BFT");
			Runner r=null;
			do{
				r=new Runner(reg,reg.getSecurityConfiguration(),b,this);
				r.setProperties(p);
				r.setAsyncMode(true);
				r.run();
				state=b.getProperty("state");
				Thread.sleep(500);
			}while(!"FINISHED".equals(state));
			//check preferred protocols
			List<ProtocolType.Enum> pp=r.getPreferredProtocols();
			assertNotNull(pp);
			assertTrue(pp.contains(ProtocolType.UFTP));
		}catch(Exception  e){
			e.printStackTrace();
			fail();
		}
	}

	@Test
	public void testAsyncWithRequestName(){
		try{
			EndpointReferenceType regEPR=WSServerUtilities.makeEPR(Registry.REGISTRY_SERVICE, 
					RegistryCreator.DEFAULT_REGISTRY_ID, uas.getKernel());
			EndpointReferenceType tsfEPR=WSServerUtilities.makeEPR(UAS.TSF, "default_target_system_factory", uas.getKernel());
			TSFClient tsf=new TSFClient(tsfEPR,uas.getKernel().getClientConfiguration());
			String address=tsf.createTSS().getEPR().getAddress().getStringValue();
			message("Created tss at "+address);
			
			RegistryClient reg=new RegistryClient(regEPR,uas.getKernel().getClientConfiguration());
			Builder b=new Builder().
				setApplication("Date", "1.0").
				writeOutputTo("target/testdata");
			b.setProperty("requestName", "mytestjob");
			String state=null;
			do{
				Runner r=new Runner(reg,reg.getSecurityConfiguration(),b,this);
				r.setAsyncMode(true);
				r.run();
				state=b.getProperty("state");
				Thread.sleep(500);
			}while(!"FINISHED".equals(state));
			
		}catch(Exception  e){
			e.printStackTrace();
			fail();
		}
	}
	
	@Test
	public void testSync(){
		try{
			EndpointReferenceType regEPR=WSServerUtilities.makeEPR(Registry.REGISTRY_SERVICE, 
					RegistryCreator.DEFAULT_REGISTRY_ID, uas.getKernel());
			EndpointReferenceType tsfEPR=WSServerUtilities.makeEPR(UAS.TSF, "default_target_system_factory", uas.getKernel());
			TSFClient tsf=new TSFClient(tsfEPR, uas.getKernel().getClientConfiguration());
			String address=tsf.createTSS().getEPR().getAddress().getStringValue();
			message("Created tss at "+address);
			
			RegistryClient reg=new RegistryClient(regEPR, uas.getKernel().getClientConfiguration());
			Builder b=new Builder().
				setApplication("Date", "1.0").
				writeOutputTo("target/testdata");
			String state=null;
			Runner r=new Runner(reg,reg.getSecurityConfiguration(),b,this);
			r.setAsyncMode(false);
			r.run();
			state=b.getProperty("state");
			assertTrue(Runner.FINISHED.equals(state));
			
		}catch(Exception  e){
			e.printStackTrace();
			fail();
		}
	}
	
	
}
