package de.fzj.unicore.ucc.bes;

import org.apache.commons.cli.OptionBuilder;
import org.ggf.schemas.bes.x2006.x08.besFactory.GetActivityStatusResponseType;
import org.ggf.schemas.bes.x2006.x08.besFactory.GetActivityStatusesDocument;
import org.ggf.schemas.bes.x2006.x08.besFactory.GetActivityStatusesResponseDocument;
import org.ggf.schemas.bes.x2006.x08.besFactory.GetActivityStatusesType;
import org.ggf.schemas.bes.x2006.x08.besFactory.GetFactoryAttributesDocumentResponseDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;




public class ListActivities extends BESActionBase {

	boolean printUrlFlag, printDetailed;
	
	@SuppressWarnings("all")
	@Override
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_DETAILED_LONG)
						.withArgName("Long Description").withDescription(
								"Long Description").isRequired(false)
						.create(OPT_DETAILED));
	}

	@Override
	public void process() {
		super.process();
		listActivities();
	}

	

	
	protected void listActivities() {

		printDetailed = getCommandLine().hasOption(OPT_DETAILED);
		
		try {
			
			verbose("Getting job info...");
			GetFactoryAttributesDocumentResponseDocument response = factory.getFactoryAttributesDocument();
			EndpointReferenceType[] eprs = response.getGetFactoryAttributesDocumentResponse().getFactoryResourceAttributesDocument().getActivityReferenceArray();
			
			if(eprs.length == 0){
				message("No activities in BESFactory: "+factory.getUrl());
				return;
			}
			
			GetActivityStatusesDocument gStatsDoc = GetActivityStatusesDocument.Factory.newInstance();
			GetActivityStatusesType gStatsType = gStatsDoc.addNewGetActivityStatuses();
			GetActivityStatusesResponseDocument resStats = null;
			
			
			if(printDetailed){
				verbose("Printing jobs with status information...");
				gStatsDoc = GetActivityStatusesDocument.Factory.newInstance();
				gStatsType = gStatsDoc.addNewGetActivityStatuses();
				gStatsType.setActivityIdentifierArray(response.getGetFactoryAttributesDocumentResponse().getFactoryResourceAttributesDocument().getActivityReferenceArray());
				resStats = factory.getActivityStatuses(gStatsDoc);
			}
			
			
			GetActivityStatusResponseType getActStatResType;		
	
			for (int i = 0; i < eprs.length; i++) {
				if(printDetailed){
					if(resStats != null){
						getActStatResType = resStats.getGetActivityStatusesResponse().getResponseArray()[i];
						String add = getActStatResType.getActivityIdentifier().getAddress().getStringValue();
						String state = getActStatResType.getActivityStatus().getState().toString();
						message(add +" -- "+state);
					}
					else{
						message("Couldn't find activity statuses");
					}
				}
				else
					message(eprs[i].getAddress().getStringValue());
			}
			
		} catch (Exception e) {
			error("Couldn't get Activity References", e);
			endProcessing(1);
		}
	}

	@Override
	public String getDescription() {
		return "list jobs running on BES.";
	}
	
	
	@Override
	public String getName() {
		return "bes-list-jobs";
	}
	
	@Override
	public String getCommandGroup() {
		return BESUCCConstants.BES_COMMAND_GROUP;
	}
	
}
