package de.fzj.unicore.ucc.bes;

import static org.junit.Assert.fail;

import java.io.File;

import org.apache.commons.io.FileUtils;
import org.junit.AfterClass;
import org.junit.BeforeClass;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.UCC;

/**
 * starts an embedded UNICORE/X (including BES) for testing
 * 
 * @author schuller
 */
public class EmbeddedTestBase implements MessageWriter {

	protected static boolean haveInit=false;

	protected static UAS uas;

	public boolean isVerbose(){return true;}

	public static void setUp(String config) throws Exception{
		try{
			if(!haveInit){
				System.out.println("Starting embedded UAS server, this takes a while...");
				FileUtils.deleteQuietly(new File("target","data"));
				uas=new UAS(config);
				uas.startSynchronous();
				UCC.unitTesting=true;
				haveInit=true;
			}
		}catch(Exception ex){
			ex.printStackTrace();
			throw ex;
		}
	}

	@BeforeClass
	public static synchronized void setUp()throws Exception{
		setUp("src/test/resources/uas/uas.config");
	}
	
	@AfterClass	
	public static synchronized void tearDown()throws Exception{
		if(uas==null)return;
		System.out.println("Stopping embedded UNICORE/X server.");
		try{
			uas.getKernel().shutdown();
		}catch(Exception e){
			e.printStackTrace();
			fail();
		}
	}

	public void verbose(String message){
		System.out.println("[ucc testing] "+message);
	}

	public void message(String message){
		System.out.println("[ucc testing] "+message);
	}

	public void error(String message, Throwable cause){
		System.err.println("[ucc testing] "+message);
		cause.printStackTrace();
	}

}
