package de.fzj.unicore.ucc.testsuite;

import de.fzj.unicore.uas.client.IRegistryQuery;
import eu.unicore.util.httpclient.IClientConfiguration;
import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.Command;
import org.apache.commons.cli.ParseException;
import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.testsuite.util.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.CommandLine;
import java.util.Map.Entry;

public class UccCommand {

	//injected by calling UCC run-groovy wrapper
	Properties properties
    Options options
	CommandLine commandLine
	IRegistryQuery registry
	IClientConfiguration securityProperties
	
	//base directory
	String baseDir;
	
	//test spec location
	String pathToTestSpec;
	
	//test spec arguments	
	String testType, testDescription, uccCommand
	File uccOutputDir
	def testArgs, uccArgs
	
	//pre/post processing scripts and their results
	def preScripts=[]
	def postScripts=[]
	def logPreScripts=[]
	def logPostScripts=[]
	
	JSONObject testSpec;

	public UccCommand(String pathToTestSpec) {
		this.baseDir=new File(pathToTestSpec).absoluteFile.parentFile.absolutePath
	    System.setProperty("user.dir",baseDir)
		this.pathToTestSpec=pathToTestSpec
	    testSpec = Utilities.createJsonObject(pathToTestSpec)
		println "UCC command in "+baseDir
		init()
	}

	// reads in test spec
	void init() {
		testType = testSpec.optString("Type")
		testDescription = testSpec.optString("Description")
		def outDir=testSpec.optString("UccOutputDir")
		if(""==outDir)outDir="test-output"
		uccOutputDir = new File(outDir).absoluteFile
		if(!uccOutputDir.exists())uccOutputDir.mkdir()
		
		//delete files of previous ucc executions
		//Utilities.deleteFiles(new File(uccOutputDir).listFiles())

		preScripts = Script.build(testSpec.optJSONArray("Pre"))
        postScripts = Script.build(testSpec.optJSONArray("Post"))
	}

	TestResult run() {
		def startTime=System.currentTimeMillis()
		preProcess()
		prepare()
		def exitCode=process()
		if(exitCode==0){
		   postProcess()
		}
		def result=new TestResult()
		result.log=printResults()
		result.test=pathToTestSpec
		result.description=testDescription
		result.timeElapsed=(System.currentTimeMillis()-startTime)/1000
		if(exitCode!=0)result.success=Boolean.FALSE
		return result
	}


	void prepare() {
		
		uccCommand = testSpec.optString("UccCommand")
		uccArgs = Utilities.JSONArrayToList(testSpec, "UccArguments")
		
		// assemble ucc arguments
		testArgs = []
		testArgs += uccCommand
		testArgs += "-o"
		testArgs += uccOutputDir
		testArgs += uccArgs
		
	}
	
	// execute 'pre' scripts
	void preProcess() {
	
	    preScripts.each{
	        Binding binding = it.binding		
	    	binding.setVariable("outputDir", uccOutputDir)
	    	binding.setVariable("baseDir", baseDir)
	    	String log=it.run()
	    	if(log!=null && !log.equalsIgnoreCase("")) {
				logPreScripts += "\n["+it.script.absolutePath+"] "+log
			}
	    }
	 }


	
	int process() {
		try{
		    UCC.unitTesting=true //UCC should not do System.exit() in case of errors
		    //create correct type of args vector (need String[] for UCC)
			String[]args=new String[testArgs.size()]
			for(int i=0;i<args.length;i++){
				args[i]=String.valueOf(testArgs[i])
			}
		    // create ucc command and execute it
		    Command cmd=UCC.initCommand(args, false)		
		    if(properties!=null)cmd.setProperties(properties)			
		    cmd.process()
		    cmd.postProcess()
		    UCC.unitTesting=false
		    return 0
		}
		catch(ParseException pex){
			//System.out.println("Error parsing commandline arguments: "+pex.getMessage());
		}
		catch(EndProcessingException epe){
			return 1
		}
		
		
	}

	// execute 'post' scripts
	void postProcess() {
	
	   postScripts.each {
	        Binding binding = it.binding		
	    	binding.setVariable("outputDir", uccOutputDir)
	    	binding.setVariable("baseDir", baseDir)
	    	String log=it.run()
	    	if(log!=null && !log.equalsIgnoreCase("")) {
				logPostScripts += "\n["+it.script.absolutePath+"] "+log
			}
	    }
	
	}


    /*
    * collect test description and gather any error messages from the pre/post scripts.
    * the results are returned
    */
	String printResults() {
		StringBuilder theLog = new StringBuilder()
		
		theLog.append("Test type: "+testType)
		theLog.append("\nTest description: "+testDescription)
		
		if(logPreScripts.size()>0){
		  theLog.append("\n\n*** PRE SCRIPTS ***\n")
		  for(String log:logPreScripts) {
			  theLog.append(log)
			  theLog.append("\n\n")
		  }
		}
		
		if(logPostScripts.size()>0){
		  theLog.append("\n\n*** POST SCRIPTS ***\n")
		  for(String log:logPostScripts) {
			  theLog.append(log)
			  theLog.append("\n\n")
		  }
        }
        
		return theLog.toString()
	}
}