package de.fzj.unicore.ucc.testsuite.util;

import org.json.JSONObject;
import org.json.JSONArray;

public class Utilities {
	
	static String readFileAsString(String pathToFile) throws Exception {
		return new File(pathToFile).text
	}

	static String readFileAsString(File file) throws Exception {
		return file.text
	}

	static File[] listFilesByExtension(File pathToDir, String extension) {
		File[] files =  pathToDir.listFiles().findAll { it.name.toLowerCase().endsWith("."+extension) } ;
		return files;
	}


	static JSONObject createJsonObject(String pathToTestCase) throws Exception{
		return new JSONObject(new File(pathToTestCase).text)
	}

	static File getCurrentFile(File[] files) {
		return (File)(files.sort(){ it.lastModified() }.reverse()[0])
	}

	static void deleteFiles(File[] files) {
		for(File f:files) {
			f.delete();
		}
	}

	static String[] JSONArrayToStringArray(JSONObject object, String key) {
		def engine=new groovy.text.SimpleTemplateEngine()
		JSONArray jsonArray = object.optJSONArray(key)
		String[] stringArray
		if(jsonArray!=null) {
			stringArray = new String[jsonArray.length()]
			for(int i=0; i<jsonArray.length(); i++) {
			    //use template to have params resolved
			    def template=engine.createTemplate(jsonArray.optString(i))
				stringArray[i] = template.make([:]).toString()
			}
			return stringArray;
		}

		else
			return new String[0];
	}

    static List JSONArrayToList(JSONObject object, String key) {
        def engine=new groovy.text.SimpleTemplateEngine()
		JSONArray jsonArray = object.optJSONArray(key)
		def result=[]
		if(jsonArray!=null) {
			for(int i=0; i<jsonArray.length(); i++) {
				//use template to have params resolved
			    def template=engine.createTemplate(jsonArray.optString(i))
				result+=template.make([:]).toString()
			}
		}
		return result
	}

	
	static File getStdOut(File outputDir) throws NullPointerException {
		File[] files = Utilities.listFilesByExtension(outputDir, "stdout");
		File f = null;		
		f = Utilities.getCurrentFile(files);
		return f;
	}

	static String getJobUID(File outputDir) {
		File[] files = listFilesByExtension(outputDir, "stdout");
		File f = Utilities.getCurrentFile(files);		
		return f.getName().replace(".stdout", "");
	}

	static File[] getJobExports(File outputDir, String[] jobExportFileNames) {		
		File[] jobExportFiles = new File[jobExportFileNames.length];
		for(int i=0; i<jobExportFileNames.length; i++) {
			File f = new File(outputDir,getJobUID(outputDir)+"."+jobExportFileNames[i]);
			jobExportFiles[i] = f;
		}
		return jobExportFiles;
	}
	
	static writeStringToFile(String filePath, String theString) {
        new File(filePath).getAbsoluteFile() << theString
	}

	static boolean fileExists(String pathToFile) {
		File file = new File(pathToFile).getAbsoluteFile();
		return file.exists();
	}

	static boolean stringSpecified(String theString) {
		return !theString.equalsIgnoreCase("") || !theString==null;
	}
	
}