package de.fzj.unicore.ucc.workflow;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import org.chemomentum.common.impl.workflow.WorkflowManagementClient;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.actions.ActionBase;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;

/**
 * workflow control (abort etc)
 * 
 * TODO future human interaction (hold, modify, continue)
 * 
 * @author schuller
 */
public class WorkflowControl extends ActionBase {
	
	private EndpointReferenceType workflowEPR;
	private String workflowID;
	private WorkflowManagementClient workflow;
	private String cmd;
	final Map<String,String>parameters=new HashMap<String,String>();
	
	@Override
	public String getName() {
		return "workflow-control";
	}
	
	@Override
	public String getArgumentList(){
		return "abort <workflow-file-or-address> | resume <workflow-file-or-address> [key1=value1 key2=value2 ...]";
	}
	
	@Override
	public String getSynopsis(){
		return "Allows to abort or resume (if held) a workflow ";
	}

	@Override
	public String getDescription(){
		return "offers workflow control functions";
	}

	
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
	}

	@Override
	public void process(){
		super.process();
		try{
			run();
		}catch(Exception e){
			error("",e);
			endProcessing(1);
		}
	}

	//read the workflow address and the command to perform
	protected void getBasicCmdlineParams()throws Exception{
		cmd=getCommandLine().getArgs()[1];
		String arg=null;
		if(getCommandLine().getArgs().length>2)arg=getCommandLine().getArgs()[2];
		
		if(arg==null){
			try{
				arg=new BufferedReader(new InputStreamReader(System.in)).readLine();
			}catch(Exception e){
				error("Can't read workflow address from stdin.",e);
				endProcessing(1);
			}
		}
		
		File wf=new File(arg);
		if(wf.exists())
		{
			workflowEPR=EndpointReferenceType.Factory.parse(wf);
			verbose("Read Workflow EPR from <"+arg+">");
		}
		else{
			//arg is an Address
			workflowEPR=EndpointReferenceType.Factory.newInstance();
			workflowEPR.addNewAddress().setStringValue(arg);
		}
		String url=workflowEPR.getAddress().getStringValue();
		verbose("Checking workflow at "+url);
		workflow=new WorkflowManagementClient(url,workflowEPR,securityProperties);
		workflowID=WSUtilities.extractResourceID(workflow.getEPR());
	}
	
	protected void run() throws Exception{
		getBasicCmdlineParams();
		readExtraParameters();
		if("abort".equalsIgnoreCase(cmd))doAbort();
		if("resume".equalsIgnoreCase(cmd))doResume();
	}
	
	protected void readExtraParameters(){
		parameters.clear();
		int length=getCommandLine().getArgs().length;
		if(length<3)return;
		for(int i=3; i<length; i++){
			String p=getCommandLine().getArgs()[i];
			String[]split=p.split("=");
			String key=split[0];
			String value=split[1];
			verbose("Have parameter: "+key+"="+value);
			parameters.put(key, value);
		}
		//unit testing use
		lastParams=parameters;
	}
	
	protected void doAbort()throws Exception{
		verbose("Aborting workflow "+workflowEPR.getAddress().getStringValue());
		if(!UCC.unitTesting)workflow.abort();
	}
	
	protected void doResume()throws Exception{
		verbose("Resuming workflow "+workflowEPR.getAddress().getStringValue());
		if(!UCC.unitTesting)workflow.resume(parameters);
	}
	
	@Override
	public String getCommandGroup(){
		return "Workflow";
	}
	
	static Map<String,String>lastParams;
	
}
